# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import argparse
import sys
import os


def get_parser():

    parser = argparse.ArgumentParser(description="OpenBases Python Paper",
                                formatter_class=argparse.RawTextHelpFormatter,
                                add_help=False)

    # Global Options
    parser.add_argument('--debug','-d', dest="debug", 
                        help="use verbose logging to debug.", 
                        default=False, action='store_true')

    parser.add_argument('--quiet','-q', dest="quiet", 
                        help="suppress all normal output", 
                        default=False, action='store_true')

    parser.add_argument('--version', dest="version", 
                        help="show openbases python version", 
                        default=False, action='store_true')

    subparsers = parser.add_subparsers(help='description',
                                       title='actions',
                                       description='actions for openbases',
                                       dest="command", metavar='general usage')

          
    # Paper Parsing

    paper = subparsers.add_parser("get",
                                   help="extract values from a paper.md")

    paper.add_argument("cmd", nargs="*",
                        help="paper markdown file to parse", 
                        type=str)

    shell = subparsers.add_parser("shell",
                                   help="start an interactive shell with openbases paper")
    return parser


def get_subparsers(parser):
    '''get_subparser will get a dictionary of subparsers, to help with printing help
    '''

    actions = [action for action in parser._actions 
               if isinstance(action, argparse._SubParsersAction)]

    subparsers = dict()
    for action in actions:
        # get all subparsers and print help
        for choice, subparser in action.choices.items():
            subparsers[choice] = subparser

    return subparsers

    # Import logger to set
    from openbases.logger import bot
    bot.debug('Logging level %s' %level)
    import openbases

    bot.debug("OpenBases Python Version: %s" % openbases.__version__)

def version():
    '''version prints the version, both for the user and help output
    '''
    import openbases
    return openbases.__version__
    

def main(main=None):

    parser = get_parser()
    subparsers = get_subparsers(parser)

    def help(return_code=0):
        '''print help, including the software version and exit
        '''
        v = version()
        print("\nOpen Bases Paper Python [v%s]\n" %(v))
        parser.print_help()
        sys.exit(return_code)
    
    if len(sys.argv) == 1:
        help()
    try:
        # We capture all primary arguments, and take secondary to pass on
        args, options = parser.parse_known_args()
    except:
        sys.exit(0)

    # If the user wants the version
    if args.version is True:
        print(version())
        sys.exit(0)

    # Does the user want help for a subcommand?
    if args.command == 'get': from .get import main 
    elif args.command == 'shell': from .shell import main 
    else: help()

    # Pass on to the correct parser
    if args.command is not None:
        main(args=args, options=options, parser=parser)


if __name__ == '__main__':
    main()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

def get_client(quiet=False, debug=False):
    '''
       return the helper function client for OpenBases

       Parameters
       ==========
       quiet: if True, suppress most output about the client
       debug: turn on debugging mode

    '''
    from .base import Client

    Client.quiet = quiet
    Client.debug = debug

    from .paper import Paper

    # Actions
    Client.paper = Paper

    # Initialize
    cli = Client()
    return cli

Client = get_client()



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
from openbases.utils import read_yaml
import os
import re
import sys
  
class Journal:
    joss = 'joss'
    rse = 'rse'

class Author:
    '''an Author holds a name, orcid id, and affiliation'''
    def __init__(self, 
                 name, 
                 orcid, 
                 affiliation):
        self.name = name
        self.orcid = orcid
        self.affiliation = affiliation


class Paper:

    def __init__(self, filename, quiet=False):

        self._check_inputs(filename)
        self.metadata = read_yaml(filename, quiet=quiet)

    def __str__(self):
        return "<paper.md: %s>" % self.filename

    def __repr__(self):
        return self.__str__()

    def __contains__(self, value):
        return value in self.metadata

    def _check_inputs(self, filename):
        '''check to make sure that filename exists
           Parameters
           ==========
           filename: the markdown file to parse
        '''
        if not os.path.exists(filename):
            bot.exit('Cannot find %s' % filename)
        self.filename = os.path.abspath(os.path.realpath(filename))


    def get(self, key, quiet=True, sep=',', field=None):
        '''return a key from the yaml, default is silent (no print) if doesn't
           exist. If the yaml item is a list with different subfields, then
           field must also be defined.
        '''
        # If the arg is of format arg:field will return field from list
        key = key.split(':')
        if len(key) > 1:
            field = key[1]

        key=key[0]

        if key in self.metadata:
    
            value = self.metadata[key]

            if isinstance(value, (tuple, list)):

                # If the first entry is a dict
                if isinstance(value[0], dict):
                    values = []

                    for entry in value:
                        if field in entry and field is not None:
                            if entry[field]:
                                values.append(entry[field])
                    value = values

                print(sep.join(value))
            else:
                print(self.metadata[key])



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.logger import bot
from subprocess import (
    Popen,
    PIPE,
    STDOUT
)
import errno
import json
import os
import re
import shutil
import sys
import yaml

################################################################################
## FOLDER OPERATIONS ###########################################################
################################################################################


def mkdir_p(path):
    '''mkdir_p attempts to get the same functionality as mkdir -p
    :param path: the path to create.
    '''
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            bot.error("Error creating path %s, exiting." % path)
            sys.exit(1)


def get_installdir():
    return os.path.dirname(os.path.abspath(__file__))

def find_subdirectories(basepath):
    '''
    Return directories (and sub) starting from a base
    '''
    directories = []
    for root, dirnames, filenames in os.walk(basepath):
        new_directories = [d for d in dirnames if d not in directories]
        directories = directories + new_directories
    return directories

    
def find_directories(root,fullpath=True):
    '''
    Return directories at one level specified by user
    (not recursive)
    '''
    directories = []
    for item in os.listdir(root):
        # Don't include hidden directories
        if not re.match("^[.]",item):
            if os.path.isdir(os.path.join(root, item)):
                if fullpath:
                    directories.append(os.path.abspath(os.path.join(root, item)))
                else:
                    directories.append(item)
    return directories

 
def copy_directory(src, dest, force=False):
    ''' Copy an entire directory recursively
    '''
    if os.path.exists(dest) and force is True:
        shutil.rmtree(dest)

    try:
        shutil.copytree(src, dest)
    except OSError as e:
        # If the error was caused because the source wasn't a directory
        if e.errno == errno.ENOTDIR:
            shutil.copy(src, dest)
        else:
            bot.error('Directory not copied. Error: %s' % e)
            sys.exit(1)




################################################################################
## FILE OPERATIONS #############################################################
################################################################################


def write_file(filename, content, mode="w"):
    '''write_file will open a file, "filename" and write content, "content"
    and properly close the file
    '''
    with open(filename, mode) as filey:
        filey.writelines(content)
    return filename


def write_json(json_obj, filename, mode="w", print_pretty=True):
    '''write_json will (optionally,pretty print) a json object to file
    :param json_obj: the dict to print to json
    :param filename: the output file to write to
    :param pretty_print: if True, will use nicer formatting
    '''
    with open(filename, mode) as filey:
        if print_pretty:
            filey.writelines(
                json.dumps(
                    json_obj,
                    indent=4,
                    separators=(
                        ',',
                        ': ')))
        else:
            filey.writelines(json.dumps(json_obj))
    return filename


def read_file(filename, mode="r", readlines=True):
    '''write_file will open a file, "filename" and write content, "content"
    and properly close the file
    '''
    with open(filename, mode) as filey:
        if readlines is True:
            content = filey.readlines()
        else:
            content = filey.read()
    return content


def read_json(filename, mode='r'):
    '''read_json reads in a json file and returns
    the data structure as dict.
    '''
    with open(filename, mode) as filey:
        data = json.load(filey)
    return data


################################################################################
## YAML ########################################################################
################################################################################


def read_yaml(filename, mode='r', quiet=False):
    '''read a yaml file, only including sections between dashes
    '''
    metadata = {}
    with open(filename, mode) as stream:
        stream = stream.read()

    # Read yaml section
    section = stream.split('---')[1]
    docs = yaml.load_all(section)
    for doc in docs:
        if isinstance(doc, dict):
            for k,v in doc.items():
                if not quiet:
                    print('%s: %s' %(k,v))
                metadata[k] = v
    return metadata




################################################################################
# environment / options
################################################################################

def convert2boolean(arg):
    '''convert2boolean is used for environmental variables
    that must be returned as boolean'''
    if not isinstance(arg, bool):
        return arg.lower() in ("yes", "true", "t", "1", "y")
    return arg


def getenv(variable_key, default=None, required=False, silent=True):
    '''getenv will attempt to get an environment variable. If the variable
    is not found, None is returned.
    :param variable_key: the variable name
    :param required: exit with error if not found
    :param silent: Do not print debugging information for variable
    '''
    variable = os.environ.get(variable_key, default)
    if variable is None and required:
        bot.error("Cannot find environment variable %s, exiting." %variable_key)
        sys.exit(1)

    if not silent:
        if variable is not None:
            bot.verbose2("%s found as %s" %(variable_key,variable))
        else:
            bot.verbose2("%s not defined (None)" %variable_key)

    return variable



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `openbases` package."""


import unittest
import os
import sys

from openbases.main import Client
test_dir = os.path.abspath(os.path.dirname(__file__))

class TestOpenBases(unittest.TestCase):
    """Tests for `openbases` package."""

    def setUp(self):
        self.cli = Client
        self.md = '%s/paper/paper.md' %test_dir
        self.paper = self.cli.paper(self.md)

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_load(self):
        """Test that the markdown is loaded fully"""
        fields = ['tags','authors','affiliations','date', 'bibliography']
        for field in fields:
            assert (field in self.paper)



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

def parse_verbosity(self, args):
    '''parse_verbosity will take an argument object, and return the args
       passed (from a dictionary) to a list

       Parameters
       ==========
       args: the argparse argument objects

    '''

    flags = []

    if args.silent is True:
       flags.append('--silent')
    elif args.quiet is True:
        flags.append('--quiet')
    elif args.debug is True:
       flags.append('--debug')
    elif args.verbose is True:
       flags.append('-' + 'v' * args.verbose)

    return flags



# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

from openbases.main.base.logger import println
from openbases.utils import run_command as run_cmd
from openbases.logger import bot

import subprocess
import json
import sys
import os
import re



def init_command(self, action, flags=None):
    '''return the initial Singularity command with any added flags.
        
       Parameters
       ==========
       action: the main action to perform (e.g., build)
       flags: one or more additional flags (e.g, volumes) 
              not implemented yet.
    '''
    cmd = ['singularity', action ]

    if self.quiet is True:
        cmd.insert(1, '--quiet')
    if self.debug is True:
        cmd.insert(1, '--debug')

    return cmd


def run_command(self, cmd, sudo=False, capture=True):
    '''run_command is a wrapper for the global run_command, checking first
       for sudo and exiting on error if needed. The message is returned as
       a list of lines for the calling function to parse, and stdout uses
       the parent process so it appears for the user.

       Parameters
       ==========
       cmd: the command to run
       sudo: does the command require sudo?
       On success, returns result. Otherwise, exists on error

    '''
    result = run_cmd(cmd, sudo=sudo, capture=capture, quiet=self.quiet)
    message = result['message']
    return_code = result['return_code']
        
    if result['return_code'] == 0:
        if len(message) == 1:
            message = message[0]
        return message

    if self.quiet is False:
        bot.error("Return Code %s: %s" %(return_code,
                                         message))




# Copyright (c) 2018, Vanessa Sochat All rights reserved.
# See the LICENSE in the main repository at:
#    https://www.github.com/openbases/openbases-python

import os

def init_level(self, quiet=False):
    '''set the logging level based on the environment
        
       Parameters
       ==========
       quiet: boolean if True, set to quiet. Gets overriden by environment
              setting, and only exists to define default

    '''
        
    if os.environ.get('MESSAGELEVEL') == "QUIET":
        quiet = True

    self.quiet = quiet



def println(self, output, quiet=False):
    '''print will print the output, given that quiet is not True. This
       function also serves to convert output in bytes to utf-8

       Parameters
       ==========
       output: the string to print
       quiet: a runtime variable to over-ride the default.

    '''
    if isinstance(output,bytes):
        output = output.decode('utf-8')
    if self.quiet is False and quiet is False:
        print(output)



from .fileio import ( 
    mkdir_p, 
    write_file, 
    write_json,
    read_file, 
    read_json,
    read_yaml
)

from .web import clone

from .terminal import ( 
    get_installdir,
    stream_command,
    run_command
)
